@@ -34,6 +34,7 @@ import pyworkflow.utils as pwutils
 import pyworkflow as pw
 from sphire.constants import *
 
+__version__ = '3.0.4'
 _logo = "sphire_logo.png"
 _references = ['Wagner2019']
 _sphirePluginDir = os.path.dirname(os.path.abspath(__file__))
@@ -41,9 +42,12 @@ _sphirePluginDir = os.path.dirname(os.path.abspath(__file__))
 
 class Plugin(pwem.Plugin):
 
+    _url = 'https://github.com/scipion-em/scipion-em-sphire'
+
     @classmethod
     def _defineVariables(cls):
         # CRYOLO do NOT need EmVar because it uses a conda environment.
+        cls._defineVar(CRYOLO_ENV_ACTIVATION_CPU, DEFAULT_ACTIVATION_CMD_CPU)
         cls._defineVar(CRYOLO_ENV_ACTIVATION, DEFAULT_ACTIVATION_CMD)
         cls._defineEmVar(CRYOLO_GENMOD_VAR, CRYOLO_GENMOD_DEFAULT)
         cls._defineEmVar(CRYOLO_GENMOD_NN_VAR, CRYOLO_GENMOD_NN_DEFAULT)
@@ -51,8 +55,11 @@ class Plugin(pwem.Plugin):
         cls._defineEmVar(CRYOLO_NS_GENMOD_VAR, CRYOLO_NS_GENMOD_DEFAULT)
 
     @classmethod
-    def getCryoloEnvActivation(cls):
-        activation = cls.getVar(CRYOLO_ENV_ACTIVATION)
+    def getCryoloEnvActivation(cls, useCpu=False):
+        if useCpu:
+            activation = cls.getVar(CRYOLO_ENV_ACTIVATION_CPU)
+        else:
+            activation = cls.getVar(CRYOLO_ENV_ACTIVATION)
         scipionHome = pw.Config.SCIPION_HOME + os.path.sep
 
         return activation.replace(scipionHome, "", 1)
@@ -87,6 +94,7 @@ class Plugin(pwem.Plugin):
     @classmethod
     def defineBinaries(cls, env):
         cls.addCryoloPackage(env, CRYOLO_DEFAULT_VER_NUM, default=bool(cls.getCondaActivationCmd()))
+        cls.addCryoloPackage(env, CRYOLO_DEFAULT_VER_NUM, default=False, useCpu=True)
         url = "wget ftp://ftp.gwdg.de/pub/misc/sphire/crYOLO-GENERAL-MODELS/"
 
         env.addPackage(CRYOLO_GENMOD, version=CRYOLO_GENMOD_201910,
@@ -138,10 +146,10 @@ class Plugin(pwem.Plugin):
         return neededProgs
 
     @classmethod
-    def addCryoloPackage(cls, env, version, default=False):
-
-        CRYOLO_INSTALLED = 'cryolo_%s_installed' % version
-        ENV_NAME = getCryoloEnvName(version)
+    def addCryoloPackage(cls, env, version, default=False, useCpu=False):
+        archFlag = 'CPU' if useCpu else ''
+        CRYOLO_INSTALLED = 'cryolo%s_%s_installed' % (archFlag, version)
+        ENV_NAME = getCryoloEnvName(version, useCpu)
         # try to get CONDA activation command
         installationCmd = cls.getCondaActivationCmd()
 
@@ -155,7 +163,8 @@ class Plugin(pwem.Plugin):
         installationCmd += 'conda activate %s &&' % ENV_NAME
 
         # Install downloaded code
-        installationCmd += 'pip install cryolo[gpu]==%s &&' % version
+        installationCmd += ('pip install cryolo[%s]==%s &&'
+                            % ('cpu' if useCpu else 'gpu', version))
 
         # Flag installation finished
         installationCmd += 'touch %s' % CRYOLO_INSTALLED
@@ -164,7 +173,7 @@ class Plugin(pwem.Plugin):
 
         envPath = os.environ.get('PATH', "")  # keep path since conda likely in there
         installEnvVars = {'PATH': envPath} if envPath else None
-        env.addPackage('cryolo', version=version,
+        env.addPackage('cryolo'+archFlag, version=version,
                        tar='void.tgz',
                        commands=cryolo_commands,
                        neededProgs=cls.getDependencies(),
@@ -172,9 +181,9 @@ class Plugin(pwem.Plugin):
                        vars=installEnvVars)
 
     @classmethod
-    def runCryolo(cls, protocol, program, args, cwd=None):
+    def runCryolo(cls, protocol, program, args, cwd=None, useCpu=False):
         """ Run crYOLO command from a given protocol. """
         fullProgram = '%s %s && %s' % (cls.getCondaActivationCmd(),
-                                       cls.getCryoloEnvActivation(), program)
+                                       cls.getCryoloEnvActivation(useCpu), program)
         protocol.runJob(fullProgram, args, env=cls.getEnviron(), cwd=cwd,
                         numberOfMpi=1)
